//// ADC configuration and calibration
/*----------------------------------------------------------------------------
 * Name:    ADC.c
 * Purpose: Freescale MK60X512VMD100 I2S Audio using TWR-AUDIO SGTL5000
 *----------------------------------------------------------------------------
//// CODE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
//// WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
//// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
//// BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
//// DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
//// PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
//// BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
//// ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
//// DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
//// INFRINGEMENT.
 *----------------------------------------------------------------------------*/
 
#include <MK60F12.H>
#include "ADC.h"

#define __ADC_IRQ 1      	/// 0=polling/blocking function, 1=interrupt

#if __ADC_IRQ
static uint16_t last;   	/// Last converted value        
        
#endif


////////////////////////////////////////////////////////////////////
// Initialize ADC1 for POT
////////////////////////////////////////////////////////////////////
void ADC1_Config(void) {

	/// Enable Clock to ADC1 
  SIM->SCGC3    	|= SIM_SCGC3_ADC1_MASK;
	/// bus clock/2 & singled-ended 16bit mode & div by 4
	ADC1->CFG1 			= ADC_CFG1_ADICLK(1) | ADC_CFG1_MODE(3) | ADC_CFG1_ADIV(2); 
	/// select channel A & high speed mode
	ADC1->CFG2 			= !ADC_CFG2_MUXSEL_MASK;
	/// software trigger & VREFH/VREFL
	ADC1->SC2 			= ADC_SC2_REFSEL(0);
	/// single shot mode
	ADC1->SC3 			&= ~ADC_SC3_ADCO_MASK;	
	/// hw ave enabled with 4 averages
// 	ADC1->SC3 			= ADC_SC3_ADCO_MASK | ADC_SC3_AVGS(0) | ADC_SC3_AVGE_MASK;

#if __ADC_IRQ
  NVIC_EnableIRQ(ADC1_IRQn);				/// Enable ADC Interrupt
	ADC1->SC1[0] = ADC_SC1_AIEN_MASK | ADC_SC1_ADCH(20);
#endif          
}

////////////////////////////////////////////////////////////////////
// Calibrate ADC1
////////////////////////////////////////////////////////////////////
void ADC1_Calibrate(void) {

	uint16_t cal_temp = 0;
	
	do{
		ADC1->SC3 |= ADC_SC3_CALF_MASK; 						/// clear any existing calibration failed flag
		ADC1->SC3 |= ADC_SC3_CAL_MASK; 							/// Start Calibration
		while(ADC1->SC3 & ADC_SC3_CAL_MASK){} 			/// wait for calibration to finish
	} while(ADC1->SC3 & ADC_SC3_CALF_MASK == 1); 	/// if there was a fault during calibration return to top and recal

	/// Clear 16b variable 
	cal_temp = 0;

	/// Add results CLP0-4 and CLPS to the variable
	cal_temp += ADC1->CLP0;
	cal_temp += ADC1->CLP1;
	cal_temp += ADC1->CLP2;
	cal_temp += ADC1->CLP3;
	cal_temp += ADC1->CLP4;
	cal_temp += ADC1->CLPS;

	/// Divide the variable by two
	cal_temp = cal_temp >> 1;
	/// Set the MSB of the variable
	cal_temp |= 0x8000;

	/// Store in plus-side gain cal reg
	ADC1->PG = cal_temp;

	/// Repeat for minus-side gain cal value
	/// Clear 16b variable
	cal_temp = 0;

	/// Add results CLP0-4 and CLMS to the variable.
	cal_temp += ADC1->CLM0;
	cal_temp += ADC1->CLM1;
	cal_temp += ADC1->CLM2;
	cal_temp += ADC1->CLM3;
	cal_temp += ADC1->CLM4;
	cal_temp += ADC1->CLMS;

	/// Divide the variable by two.
	cal_temp = cal_temp >> 1;
	/// Set the MSB of the variable.
	cal_temp |= 0x8000;

	/// Store value in minus-side gain cal reg
	ADC1->MG = cal_temp;

}

//////////////////////////////////////////////
//	Blocking function to return ADC1 
//	conversion
/////////////////////////////////////////
uint16_t get_ADC1_Blocking(void) {
		
	uint16_t un16_ADC1_RA= 0;
	
	ADC1->SC1[0] = !ADC_SC1_AIEN_MASK | ADC_SC1_ADCH(20);
	while((ADC1->SC1[0] & ADC_SC1_COCO_MASK)!= 0){} 
				un16_ADC1_RA = ADC1->R[0];

	return un16_ADC1_RA;
	
}


//////////////////////////////////////////////
//	Non-Blocking function to return last ADC1 
//	conversion
/////////////////////////////////////////
#if __ADC_IRQ
uint16_t get_ADC1_int(void) {

	ADC1->SC1[0] |= 20;			/// start sw conversion
	
  return (last);   /// Return converted value
  
}
#endif


//////////////////////////////////////////////
//	IRQ handler to store last ADC1 
//	conversion and done flag
/////////////////////////////////////////
#if __ADC_IRQ
void ADC1_IRQHandler (void) {

  if (ADC1->SC1[0] & ADC_SC1_COCO_MASK) 
	{
    last = ADC1->R[0];

  }
}
#endif




