/*----------------------------------------------------------------------------
 * Name:    Talkthrough.c
 * Purpose: Freescale MK60X512VMD100 I2S Audio using TWR-AUDIO SGTL5000
 *----------------------------------------------------------------------------
//// CODE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
//// WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
//// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
//// BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
//// DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
//// PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
//// BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
//// ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
//// DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
//// INFRINGEMENT.
 *----------------------------------------------------------------------------*/

#include <MK60F12.H>
#include <math.h>
#include <stdio.h>
#include "I2C.h"
#include "ADC.h"
#include "I2S.h"
#include "SGTL5000.h"

#define LED_NUM     	4       /// Number of user LEDs 
#define BYPASS				1
#define NO_BYPASS			0
#define MUTE					1
#define UNMUTE				0
#define TRUE					1
#define FALSE					0
#define STEREO				1				/// define stereo 1 or mono 0
	
///////////////////////////////////////////////////////
//	Function prototypes
//	and global variables/constants
//////////////////////////////////////////////////////
void get_audio_mag(int32_t*, int32_t, int32_t);


/// Globals for inline functions
const uint32_t led_mask[] = {1UL << 11, 1UL << 28, 1UL << 29, 1UL << 10};    
volatile uint32_t msTicks;      	/// counts 1ms timeTicks for timing/button dbounce
volatile uint32_t msTicks2;      	/// counts 1ms timeTicks for timing/button dbounce

  
////////////////////////////////////////////////////////////////////
// SysTick_Handler
////////////////////////////////////////////////////////////////////
void SysTick_Handler(void) {
  msTicks++;                  		/// increment counters for use in timekeeping
	msTicks2++;
}

////////////////////////////////////////////////////////////////////
// Initialize and config LEDs (based on origional Blink example)
////////////////////////////////////////////////////////////////////
__INLINE static void LED_Config(void) {

  SIM->SCGC5    	|= (1UL <<  9);        	/// Enable Clock to Port A  

  PORTA->PCR[11] 	= (1UL <<  8);        	/// mux as GPIO for LED
  PORTA->PCR[28] 	= (1UL <<  8);        	/// mux as GPIO for LED
  PORTA->PCR[29] 	= (1UL <<  8);        	/// mux as GPIO for LED
	PORTA->PCR[10] 	= (1UL <<  8);					/// mux as GPIO for LED
	
	/// switch LEDs off
  PTA->PDOR = (led_mask[0] | led_mask[1] | led_mask[2] | led_mask[3] );          
	/// enable Output
  PTA->PDDR = (led_mask[0] | led_mask[1] | led_mask[2] | led_mask[3] );        
}

////////////////////////////////////////////////////////////////////
// Initialize and config GPIO PTE26 and PTA19
// read pins on PTA->PDIR & (1UL << 19)  and  PTE->PDIR & (1UL << 26) 
////////////////////////////////////////////////////////////////////
__INLINE static void GPIO_Config(void) {

  SIM->SCGC5    	|= SIM_SCGC5_PORTA_MASK;        /// Enable Clock to Port A  
	SIM->SCGC5    	|= SIM_SCGC5_PORTE_MASK;       	/// Enable Clock to Port E 
	
	/// Set PTA19 and PTE26 (connected to SW1 and SW2) for GPIO functionality, falling IRQ,
  ///  and to use internal pull-ups. (pin defaults to input state)
  PORTA->PCR[19] 	=	PORT_PCR_MUX(1) | PORT_PCR_IRQC(0xA) | PORT_PCR_PE_MASK | PORT_PCR_PS_MASK;
  PORTE->PCR[26] 	=	PORT_PCR_MUX(1) | PORT_PCR_IRQC(0xA) | PORT_PCR_PE_MASK | PORT_PCR_PS_MASK;	

}

/////////////////////////////////////////////
//  Switch on LEDs
/////////////////////////////////////////////
__INLINE static void LED_On (uint32_t led) {

  PTA->PCOR   = led_mask[led];
}

/////////////////////////////////////////////
//  Switch off LEDs
/////////////////////////////////////////////
__INLINE static void LED_Off (uint32_t led) {

  PTA->PSOR   = led_mask[led];
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
//  MAIN function
//
//////////////////////////////////////////////////////////////////////////////////
int main (void) {

	/// Local Variables for main
	//-----------------------------//
	int32_t		n32_lt_audio_in 		=	0;
	int32_t		n32_rt_audio_in 		=	0;
	int32_t		n32_lt_audio_out 		=	0;
	int32_t		n32_rt_audio_out 		=	0;	
	int32_t 	n32_audio_magnatude = 0;
	int32_t 	n32_tap_count 			= 0;
	uint16_t	un16_ADC1_RA 				= 0;
	uint8_t 	un8_toggle					=	0;
	float			f_ADC_Norm					=	0.0;


	/// Initialization Funcitons
	//-----------------------------//
  SystemCoreClockUpdate();                      /// Get Core Clock Frequency
  SysTick_Config(SystemCoreClock/1000);         /// Generate interrupt each 1 ms 
  
  LED_Config();   															/// setup LED IO inline                         
  I2C0_Config();																/// setup I2C0 inline 
  I2S0_Config();																/// setup I2S0 inline
	GPIO_Config();																/// setup GPIO for PTA19 and PTE26 inputs SW1 and SW2
	ADC1_Config();																/// setup ADC1 for use with POT
	ADC1_Calibrate();															/// calibrate ADC1
	SGTL5000_Init();															/// call function to initialize SGTL5000 audio codec 24 bit 48KHz 

	SGTL5000_Bypass(NO_BYPASS);										/// dont bypass I2S
	SGTL5000_LO_Vol(0xF,0xF);											/// Set Line out to nominal based on 40*log(VAG_VAL/LO_VAGCNTRL)+15 = 15 or 0xF 	
	SGTL5000_HP_Vol(6,6);													/// Boost HP gain 0.5 dB per dev from 24
	SGTL5000_DAP_Mch_Vol(0x8000);									/// Set DAP vol default 0x8000 = 100%, 0xFFFF = 200%
	SGTL5000_DAP_Surround(0, 0);									/// Select Freescale surround off and no width
	SGTL5000_ADC_Vol(TRUE, 2, 2);									/// Select ADC 6db range reduction and rt and lf ADC volume
	
  while(1) {

		
 		/// Recieve I2S LT and RT channel data and tx LT and RT channels
		/// pass args lt_in and rt_in by ref, pass rt_out and lt_out by val 
		I2S_RxTx(&n32_rt_audio_in, &n32_lt_audio_in, n32_rt_audio_out, n32_lt_audio_out);
			
			
		/// pass audio in to audio out
		if(STEREO)
		{
		n32_lt_audio_out = n32_lt_audio_in * f_ADC_Norm; 
		n32_rt_audio_out = n32_rt_audio_in * f_ADC_Norm; 			
		}
		else
		{
		n32_lt_audio_out = n32_lt_audio_in * f_ADC_Norm; 
		n32_rt_audio_out = n32_lt_audio_out; 
		}
		
		/// Use either blocking or interrupt based adc conversion
// 		un16_ADC1_RA = get_ADC1_Blocking();
		un16_ADC1_RA = get_ADC1_int();
			
		/// convert un16_ADC1_RA value to a normalized float
		f_ADC_Norm = (float)(un16_ADC1_RA/65535.0);
			

		/// Add SGTL5000 Audio Enhancment when sw2 is pressed 	
		if(!(PTE->PDIR & (1UL << 26)) && msTicks > 350)
		{	
			msTicks = 0;
			un8_toggle = !un8_toggle;
			if(un8_toggle == 1)
			{
				SGTL5000_DAP_Bass_En(1, 6, 1);								/// bypass hpf, cut 225 Hz, boost bass
				SGTL5000_DAP_Bass_En_CNTL(5, 0);							/// set to max bass	default L/R level			
				SGTL5000_DAP_Surround(2, 3);									/// Select Freescale surround stereo with med width 
			}
			else if(un8_toggle == 0)
			{
				SGTL5000_DAP_Bass_En(0, 4, 0);								/// set to default
				SGTL5000_DAP_Bass_En_CNTL(5, 0x1F);						/// set to default
				SGTL5000_DAP_Surround(4, 0);									/// set to default
			}				
		}
		
		
		
		/// Bass magnitude on LEDs
		////////////////////////////////////////////
		get_audio_mag(&n32_audio_magnatude, n32_lt_audio_out, n32_rt_audio_out);
		n32_tap_count++;

		if(n32_tap_count==128)
		{
			n32_tap_count = 0;
			if(n32_audio_magnatude > 3800)
			{
				LED_On (0);
				LED_On (1);
				LED_On (2);
				LED_On (3);
			}
			else if(n32_audio_magnatude > 2500)
			{
				LED_Off (0);
				LED_On (1);
				LED_On (2);
				LED_On (3);
			}			
			else if(n32_audio_magnatude > 1800)
			{
				LED_Off (0);
				LED_Off (1);
				LED_On (2);
				LED_On (3);
			}					
			else if(n32_audio_magnatude > 1200)
			{
				LED_Off (0);
				LED_Off (1);
				LED_Off (2);
				LED_On (3);
			}					
			else 
			{
				LED_Off (0);
				LED_Off (1);
				LED_Off (2);
				LED_Off (3);
			}	
			n32_audio_magnatude	= 0;
			
		}


		
  } /// end while(1)
		
} /// end main


//////////////////////////////////////////////////////////////////////
/// Get the relative stereo magnatude of audio stream
//////////////////////////////////////////////////////////////////////
void get_audio_mag(int32_t* n32_audio_magnatude, int32_t n32_lt_audio_in, int32_t n32_rt_audio_in)	{	

	int32_t audio_magnatude_temp = 0;
		
	audio_magnatude_temp = (n32_lt_audio_in >> 19) + (n32_rt_audio_in >> 19);
		if(audio_magnatude_temp < 0)
			*n32_audio_magnatude += -audio_magnatude_temp;
		else
			*n32_audio_magnatude += audio_magnatude_temp;
	
	
}
	




